@@ -2,14 +2,16 @@
 @Author  : Likianta <likianta@foxmail.com>
 @Module  : excel_reader.py
 @Created : 2019-00-00
-@Updated : 2020-08-09
+@Updated : 2020-09-10
 @Version : 1.2.17
 @Desc    :
 """
+from typing import *
+
 import xlrd
 
-from ._typings import *
-
+_CellValue = Union[str, bool, int, float, None,]
+_ColValues = _RowValues = Iterable
 
 """
 假设表格为:
@@ -115,7 +117,7 @@ class ExcelReader(ColIndexLocator):
     book = None
     sheet = None
     header = None
-
+    
     # noinspection PyMissingConstructor
     def __init__(self, path: str, sheetx=0, formatting_info=False):
         """
@@ -154,7 +156,7 @@ class ExcelReader(ColIndexLocator):
         else:
             self.header = None
             self._header = None
-
+    
     # --------------------------------------------------------------------------
     
     def get_num_of_sheets(self) -> int:
@@ -168,9 +170,9 @@ class ExcelReader(ColIndexLocator):
     
     def get_num_of_cols(self) -> int:
         return self.sheet.ncols
-
+    
     # --------------------------------------------------------------------------
-
+    
     def get_header(self) -> list:
         return self.header
     
@@ -186,12 +188,12 @@ class ExcelReader(ColIndexLocator):
     
     def get_filepath(self) -> str:
         return self.filepath
-
+    
     # --------------------------------------------------------------------------
-
+    
     # noinspection PyUnboundLocalVariable
     @staticmethod
-    def _betterint(v: Any) -> CellValue:
+    def _betterint(v: Any) -> _CellValue:
         """
         在 excel 中, 所有数字皆以浮点储存. 但考虑到个人需求, 我需要将一些整数在
         python 中以 int 表示. 我将它上升为一个重要的决策. 尽管它可能带来一些负面
@@ -202,20 +204,20 @@ class ExcelReader(ColIndexLocator):
         else:
             return v
     
-    def cell_value(self, x, y) -> CellValue:
+    def cell_value(self, x, y) -> _CellValue:
         v = self.sheet.cell(x, y).value
         return self._betterint(v)
     
-    def row_values(self, rowx: int) -> RowValues:
+    def row_values(self, rowx: int) -> _RowValues:
         return [
             self._betterint(x)
             for x in self.sheet.row_values(rowx)
         ]
     
-    def row_dict(self, rowx: int) -> Dict[Union[str, int], CellValue]:
+    def row_dict(self, rowx: int) -> Dict[Union[str, int], _CellValue]:
         return dict(zip(self.header, self.row_values(rowx)))
     
-    def col_values(self, query, offset=0) -> Union[ColValues, None]:
+    def col_values(self, query, offset=0) -> Union[_ColValues, None]:
         if not isinstance(query, int):
             # str, list, tuple
             if isinstance(query, (str, bool, float)):
@@ -224,18 +226,18 @@ class ExcelReader(ColIndexLocator):
                 return None
         else:
             colx = query
-            
+        
         return [
             self._betterint(x)
             for x in self.sheet.col_values(colx)[offset:]
         ]
-
+    
     # --------------------------------------------------------------------------
     # Iterations
-
+    
     def get_range(self, offset=0):
         return range(offset, self.sheet.nrows)
-
+    
     def zip_cols(self, *cols, offset=1):
         if len(cols) == 1:
             return self.col_values(cols[0], offset)
@@ -243,14 +245,14 @@ class ExcelReader(ColIndexLocator):
             return zip(
                 *(self.col_values(x, offset) for x in cols)
             )
-
+    
     def enum_cols(self, *cols, offset=1):
         if len(cols) == 1:
             return enumerate(self.col_values(cols[0], offset), offset)
         else:
             return zip(range(offset, self.get_num_of_rows()),
                        *(self.col_values(x, offset) for x in cols))
-
+    
     # noinspection PyArgumentList
     def walk_rows(self, offset=0, fmt='list'):
         """
@@ -259,7 +261,7 @@ class ExcelReader(ColIndexLocator):
         assert fmt in ('list', 'dict')
         handle = self.row_values if fmt == 'list' else self.row_dict
         return (handle(rowx) for rowx in range(offset, self.sheet.nrows))
-
+    
     # noinspection PyArgumentList
     def enum_rows(self, offset=1, fmt='list'):
         """
