@@ -2,8 +2,8 @@
 @Author  : Likianta <likianta@foxmail.com>
 @Module  : file_combinator.py
 @Created : 2019-00-00
-@Updated : 2020-08-09
-@Version : 2.3.0
+@Updated : 2020-09-06
+@Version : 2.3.1
 @Desc    :
     使用方法:
         合并指定目录下的所有 txt 文件, 并在同目录下生成 "result.txt" 文件:
@@ -17,10 +17,11 @@
 """
 import os
 
+from lk_logger import lk
+
 from . import filesniff, read_and_write
 from .excel_reader import ExcelReader
 from .excel_writer import ExcelWriter
-from .lk_logger import lk
 
 
 def combine_txt(idir: str, trim_header=True):
@@ -98,12 +99,12 @@ def combine_excel(ifiles, ofile='', module=True, one_sheet=True):
     
     :param ifiles: 来源文件, 可以是多个, 以列表或 iterable 形式传入
     :param ofile: 目标输出文件的路径. 为空则表示在 idir 目录下生成一个 merged.
-     xlsx 文件. ofile 的后缀名只能是 '.xlsx' (当为 '.xls' 时, 会被强制转换)
+        xlsx 文件. ofile 的后缀名只能是 '.xlsx' (当为 '.xls' 时, 会被强制转换)
     :param module: 是否在合并结果中添加一个 "module" 列, 以显示分文件的来源 (文
-     件名)
+        件名)
     :param one_sheet:
         True: 将分文件的数据合并到输出文件的一个 sheet 中. 建议当分文件的字段格
-         式相同时使用.
+            式相同时使用.
         False: 将分文件的数据按照其文件名创建到输出文件的不同 sheet 中. 建议:
             (1) 当分文件的字段不同时使用;
             (2) 当分文件是不同的年份的数据, 我们想要按照年份来分放到输出文件的不
@@ -128,8 +129,8 @@ def combine_excel(ifiles, ofile='', module=True, one_sheet=True):
             writer.add_new_sheet(f'{fileno}-{filename[:29]}')
             """
             注意: 这里用 filename 作 sheet name 有隐患, 当名称过长 (超过 31 个字
-             符) 或重名时会报错, 因此我们加一个 modulex 前缀避免重名, 再裁剪至最
-             多 31 个字符.
+                符) 或重名时会报错, 因此我们加一个 modulex 前缀避免重名, 再裁剪
+                至最多 31 个字符.
             """
         
         if header is None or one_sheet is False:
