@@ -3,11 +3,11 @@
 @Module  : read_and_write.py
 @Created : 2018-08-00
 @Updated : 2020-11-22
-@Version : 1.9.0
+@Version : 1.8.6
 @Desc    :
 """
-from os.path import exists
 from contextlib import contextmanager
+from os.path import exists
 
 from ._typing import ReadAndWriteHint as Hint
 
@@ -15,29 +15,36 @@ from ._typing import ReadAndWriteHint as Hint
 @contextmanager
 def ropen(file: str, mode='r', encoding='utf-8'):
     """
-    :param file:
-    :param mode:
-    :param encoding: ['utf-8'|'utf-8-sig']
+    Args:
+        file
+        mode (['r'|'rb'])
+        encoding (['utf-8'|'utf-8-sig'])
+    
+    Returns:
+        file_handle
     """
+    # 由于大多数国内 Windows 系统默认编码为 GBK, Python 内置的 open() 函数默认使用的是它,
+    # 但在开发中经常会遇到编码问题; 所以我们明确指定编码为 UTF-8, 本函数只是为了简化此书写步骤.
+    handle = open(file, mode=mode, encoding=encoding)
     try:
-        # 由于大多数国内 Windows 系统默认编码为 GBK, Python 内置的 open() 函数默认使用的
-        # 是它, 但在开发中经常会遇到编码问题; 所以我们明确指定编码为 UTF-8, 本函数只是为了简
-        # 化此书写步骤.
-        handle = open(file, mode=mode, encoding=encoding)
         yield handle
     finally:
-        # noinspection PyUnboundLocalVariable
         handle.close()
-        
-        
+
+
 @contextmanager
 def wopen(file: str, mode='w', encoding='utf-8'):
     """
-    :param file:
-    :param mode: ['w'|'a']
-        w: 写入前清空原文件已有内容
-        a: 增量写入
-    :param encoding: ['utf-8'|'utf-8-sig']
+    Args:
+        file (str):
+        mode (['w'|'a'|'wb']):
+            w: 写入前清空原文件已有内容
+            a: 增量写入
+            wb: 以二进制字节流写入
+        encoding (['utf-8'|'utf-8-sig']):
+        
+    Returns:
+        file_handle
     """
     try:
         handle = open(file, mode=mode, encoding=encoding)
@@ -49,7 +56,8 @@ def wopen(file: str, mode='w', encoding='utf-8'):
 
 def get_num_of_lines(file: str) -> int:
     """ 该方法可以高效地读取文件一共有多少行, 支持大文件的读取.
-    REF: https://blog.csdn.net/qq_29422251/article/details/77713741
+    
+    Refer: https://blog.csdn.net/qq_29422251/article/details/77713741
     """
     with ropen(file) as f:
         return len(['' for _ in f])
@@ -57,8 +65,9 @@ def get_num_of_lines(file: str) -> int:
 
 def is_file_has_content(file: str) -> bool:
     """
-    REF: https://www.imooc.com/wenda/detail/350036?block_id=tuijian_yw
-    :return: bool.
+    Refer: https://www.imooc.com/wenda/detail/350036?block_id=tuijian_yw
+    
+    Returns (bool):
         True: file has content
         False: file is empty
     """
@@ -79,12 +88,12 @@ def read_file(file: str) -> str:
 
 def read_file_by_line(file: str, offset=0) -> list:
     """
-    IN: file: str. e.g. 'test.txt'
-        offset: int.
+    IN: file (str): e.g. 'test.txt'
+        offset (int):
             0: 表示返回完整的列表
             n: 传一个大于 0 的数字, 表示返回 list[n:]. (ps: 如果该数字大于列表的
                长度, python 会返回一个空列表, 不会报错)
-    OT: e.g. ['aaa', 'bbb', 'ccc', ...]
+    OT: [str, ...]
     """
     # https://blog.csdn.net/qq_40925239/article/details/81486637
     with ropen(file) as f:
@@ -95,23 +104,22 @@ def read_file_by_line(file: str, offset=0) -> list:
 def write_file(content: iter, file: str, mode='w', adhesion='\n'):
     """ 写入文件, 传入内容可以是字符串, 也可以是数组.
 
-    ARGS:
-        content: 需要写入的文本, 可以是字符串, 也可以是数组. 传入数组时, 会自动
-            将它转换为 "\n" 拼接的文本
+    Args:
+        content: 需要写入的文本, 可以是字符串, 也可以是数组. 传入数组时, 会自动将它转换为
+            "\n" 拼接的文本
         file: 写入的路径, 建议使用相对路径
-        mode: ['w'|'a'|'wb']. 写入模式, 有三种可选:
+        mode (['w'|'a'|'wb']): 写入模式, 有三种可选:
             a: 增量写入 (默认)
             w: 清空原内容后写入
             wb: 在 w 的基础上以比特流的形式写入
         adhesion: ['\n'|'\t'|...]. 拼接方式, 只有当 content 为列表时会用到, 用于将列表
             转换为文本时选择的拼接方式
-            E.g.
+            Example:
                 content = adhesion.join(content)
                 # ['a', 'b', 'c'] -> 'a\nb\nc'
 
-    参考:
-        python 在最后一行追加 - 张乐乐章 - 博客园 https://www.cnblogs.com
-            /zle1992/p/6138125.html
+    Refer:
+        python 在最后一行追加 https://www.cnblogs.com/zle1992/p/6138125.html
         python map https://blog.csdn.net/yongh701/article/details/50283689
     """
     if not isinstance(content, str):
@@ -135,30 +143,56 @@ def read_json(file: str) -> dict:
         # noinspection PyUnresolvedReferences
         from yaml import safe_load  # pip install pyyaml
         return safe_load(file)
-    raise Exception('Unknown file type! Please pass a file ends with ".json" '
-                    'or ".yaml".')
+    else:
+        raise Exception(
+            'Unknown file type! Please pass a file ends with '
+            '".json" or ".yaml".'
+        )
 
 
-def write_json(data: Hint.DumpableData, file: str):
-    # assert file.endswith('.json')
+def write_json(data: Hint.DumpableData, file: str, pretty_dump=False):
+    """
+    Args:
+        data
+        file: Ends with '.json'.
+            Note: '.yaml' needs PyYaml to be installed.
+        pretty_dump:
+            True: Output json with pretty-printed format (4 spaces each level).
+            False: The most compact representation.
+            
+    Improvements:
+        Convert `set` as `list` to make it serializable.
+        Convert `pathlib.PosixPath` as `str` to make it serializable.
+    """
     from json import dumps as _dumps
-    if isinstance(data, set):
-        data = tuple(data)
+    
+    if isinstance(data, Hint.SerialikableData):
+        data = list(data)
+        #   Note: `zip([A, B], [C, D])` will be converted to
+        #   `[(A, C), (B, D)]`
+    
     with wopen(file) as f:
-        f.write(_dumps(data, ensure_ascii=False))
-        #   ensure_ascii: json.dumps 如何输出中文:
+        f.write(_dumps(data, ensure_ascii=False, default=str,
+                       indent=None if pretty_dump is False else 4))
+        #   ensure_ascii=False: Set to False to support Chinese characters:
         #       https://www.cnblogs.com/zdz8207/p/python_learn_note_26.html
+        #   default=str: When something is not serializble, try to use its
+        #       __str__() attribute, it is useful to resolve `pathlib.PosixPath`
 
 
 # ------------------------------------------------------------------------------
 
-def loads(file: str, offset=-1) -> Hint.LoadedData:
+def loads(file: str, offset=-1, **kwargs) -> Hint.LoadedData:
     """
-    :param file:
-    :param offset: [-1|0|1|2|3|...]
-            -1: 当为负数时, 表示调用 read_file()
-            0, 1, 2, 3, ...: 表示调用 read_file_by_line(), offset 作为
-                read_file_by_line() 的相应参数传入
+    Args:
+        file:
+        offset: [-1|0|1|2|3|...]
+            -1: If it is negative number (typically -1), means calling
+                `read_file`
+            0, 1, 2, 3, ...: See `read_file_by_line`
+        kwargs: See `ExcelReader`.
+            sheetx=0
+            formatting_info=False
     """
     if file.endswith(Hint.StructFileTypes):
         return read_json(file)
@@ -170,8 +204,9 @@ def loads(file: str, offset=-1) -> Hint.LoadedData:
     elif file.endswith(Hint.BinaryFileTypes):
         if file.endswith(('.xlsx', '.xls')):
             from .excel_reader import ExcelReader
-            return ExcelReader(file)
-    raise Exception('Unsupported filetype!', file)
+            return ExcelReader(file, **kwargs)
+    else:
+        raise Exception('Unsupported filetype!', file)
 
 
 def load_list(file: str, offset=0) -> list:
@@ -183,15 +218,17 @@ def load_dict(file: str) -> dict:
         return read_json(file)
     # elif file.endswith('.txt'):
     #     return {lineno: line for (lineno, line) in enumerate(load_list(file))}
-    raise Exception('Unsupported filetype!', file)
+    else:
+        raise Exception('Unsupported filetype!', file)
 
 
-def dumps(data: Hint.DumpableData, file: str):
+def dumps(data: Hint.DumpableData, file: str, **kwargs):
     if file.endswith(Hint.StructFileTypes):
-        write_json(data, file)
+        write_json(data, file, **kwargs)
     elif file.endswith(Hint.PlainFileTypes):
-        write_file(data, file)
-    raise Exception('Unsupported filetype!', file)
+        write_file(data, file, **kwargs)
+    else:
+        raise Exception('Unsupported filetype!', file)
 
 
 # ------------------------------------------------------------------------------
@@ -207,29 +244,25 @@ def read(file: str, **kwargs):
 
 @contextmanager
 def write(file: str, data=None, **kwargs):
-    """
+    """ Yields a write handle to caller, the file will be generated after the
+        `with` block closed.
+        
+    Args:
+        file: See `dumps`.
+        data ([list|dict|set|str]): If the data type is incorrect, an Assertion
+            Error will be raised.
+        kwargs: See `dumps`.
+        
     Usage:
         with write('result.json', []) as w:
             for i in range(10):
                 w.append(i)
-        # Then when we exit the `with` block, the data will be automatically
-        # saved to 'result.json'.
+        print('See "result.json:1"')
+        # When we exit the `with` block, the data will be automatically saved to
+        # 'result.json' file.
     """
+    assert isinstance(data, (list, dict, set, str))
     try:
-        assert isinstance(data, (list, dict, set, str))
-        try:
-            yield data
-        finally:
-            dumps(data, file)
-    except AssertionError as e:
-        if data is None and file.endswith('.xlsx'):
-            from .excel_writer import ExcelWriter
-            holder = ExcelWriter(file, **kwargs)
-            try:
-                yield holder
-            finally:
-                holder.__h = 'grand_parent'
-                holder.save()
-                holder.__h = 'parent'
-        else:
-            raise e
+        yield data
+    finally:
+        dumps(data, file, **kwargs)
